/****************************************************************************/
/*                  JL-080 sample programs for Slave                        */
/*                                                                          */
/*    This file contains sample programs of                                 */
/*      (1) slv_init(void) : JL-080 initialization for MECHATROLINK-II,     */
/*      (2) slv_exchange(void) : Send/recieve data processing.              */
/*                                                                          */
/*    These sample programs are made under the following conditions;        */
/*              Slave station address : 0x41                                */
/*                 Transmission Cycle : 1 msec                              */
/*      JL-080 register start address : 0xDE000000                          */
/*                        packet size : 32 [bytes]                          */
/*                                                                          */
/*    Please refer to the following document for more detials.              */
/*       "MECHATROLINK-II JL-080 Access Driver Instructions                 */
/*                                            for Slave Station"            */
/*                                                                          */
/*    == IMPORTANT ==                                                       */
/*      The interface registers of the JL-080                               */
/*      handle unsigned short data in the sample programs.                  */
/*      When using an implementation in which unsigned short data is not    */
/*      16 bits, it will be necessary to change the part of the source code */
/*      that accesses the interface registers.                              */
/*                                                                          */
/*                                          Yaskawa Electric Corporation    */
/*                                                                          */
/****************************************************************************/

/* includes */
#include "jl080gbl.h"

/* defines */
#define LINK_SIZE			16			/* Link data size [in words] 16 means 32 bytes */
#define SLV_ST_ADDR			0x41		/* Slave station address */
#define HOST_IF_REGS_ADDR	0xfb0000	/* JL-080 register start address */

/* globals */
CHANNEL_INFO	chbuff;					/* Channel Buffer */
USHORT			lnk_sbuff[LINK_SIZE];	/* Send Buffer */
USHORT			lnk_rbuff[LINK_SIZE];	/* Receive Buffer */
USHORT			c1_t_mcyc;				/* Received transmission cycle time in MII mode[0.25us]*/

/* forward declarations */
int slv_init(void);						/* Initialize MECHATROLINK communicaiton(Setup JL-080) */
int slv_exchange(void);					/* Send/Recieve Link data in cyclic communication */



/****************************************************************************/
/*                                                                          */
/* Name :                                                                   */
/*        Sample of JL-080 Initialization procedure                         */
/* Function:                                                                */
/*       Initializing JL-080                                                */
/* Conditions:                                                              */
/*                  Protocol : MECHATROLINK-II(2:N Communication)           */
/*               Buffer mode : Fixed buffer mode                            */
/*                     Speed : 10Mbps                                       */
/*        Transmission cycle : 1ms                                          */
/*               Packet Size : 32[byte]                                     */
/*                                                                          */
/* Return Code :                                                            */
/*      OK :                            Normal End                          */
/*      ERROR_INIT_CHANNEL :            Invalid channel buffer specified    */
/*      ERROR_POLL_TIMEOUT :            Request did not completed           */
/*      ERROR_NOT_READY :               JL-080 Not Ready (STS(RDY) is zero.)*/
/*      ERROR_INVALID_PROTOCOL :        Unsupported protocol                */
/*      ERROR_INVALID_BMODE :           Unsupported buffer mode             */
/*      ERROR_POLL_TIMEOUT_REG :        P_RSTAK Request did not completed   */
/*      ERROR_POLL_TIMEOUT_PAR :        PAR_I Request did not completed     */
/*      ERROR_POLL_TIMEOUT_ENA :        Request did not completed           */
/****************************************************************************/
int slv_init(void)
{
	CHANNEL_INFO*	chbufp;		/* Channel Buffer */
	YENET_USER_PAR	usr_par;	/* Communication Parameters */
	int				ret = 0;	/* return code */

	chbufp = &chbuff;	/* Get the pointer of Channel Buffer */

/* Setup parameters of JL-080 */
	usr_par.mod = YN_MOD_PROCEL_2NS | YN_MOD_SPEED_10M 
				| YN_MOD_BMOD_STATIC | YN_MOD_TYPE_SLAVE;

	usr_par.ma = SLV_ST_ADDR;		/* My Address */
	usr_par.ma_max = 1;				/* reserve(default:1) */
	usr_par.t_mcyc = 4000;			/* Transmission cycle[0.25us] */
	usr_par.t_cyc = 0;				/* reserve(default:0) */
	usr_par.byte = 32;				/* Transimit Data Size (Bytes) */
	usr_par.dev = 0;				/* reserve(default:0) */
	usr_par.max_rtry = 0;			/* reserve(default:0) */
	usr_par.c2m_ch = 0;				/* reserve(default:0) */
	usr_par.wdt = 15;				/* Watch dog timer [10ms]; if wdt=0, WDT function disabled */

/* JL-080 Initialization */
	ret = yns_sequent_init(chbufp, (USHORT*)HOST_IF_REGS_ADDR, &usr_par, NULL);

	return ret;
}


/****************************************************************************/
/*                                                                          */
/* Name :                                                                   */
/*       Sample of send/recieve data procedure                              */
/*                                                                          */
/* Function :                                                               */
/*      Setting data in the lnk_sbuff to JL-080, and saving received data   */
/*      to the lnk_rbuff.                                                   */
/*                                                                          */
/* Return Code:                                                             */
/*      OK :                          Normal End                            */
/*      ERROR_ALT_BUFFER :            Time out of Toggle buffer             */
/*      ERROR_INVALID_ST_NO :         Invalid Channel No.                   */
/*      ERROR_RECV_DATA:              Receive error                         */
/*      ERROR_NO_SYNC:                Invalid interruption                  */
/*      ERROR_JL080_APPL:             JL-080 status error(user)             */
/*      ERROR_JL080_SYSTEM:           JL-080 status error(system)           */
/*                                                                          */
/****************************************************************************/
int slv_exchange(void)
{
	CHANNEL_INFO*	chbufp;		/* Channel Buffer */
	USHORT			ests;		/* Error Status */
	int				ret = OK;	/* return code */

	chbufp = &chbuff;	/* Set pointer of Channel Buffer */

/* Start cyclic communication */
	if((ret = yns_begin_link_acc(chbufp)) != OK)
	{
	/* Error detection */
		yns_end_link_acc(chbufp);/* Terminate cyclic communication */
		return ret;
	}

/* Get Sync-Frame status */
	if(yns_get_gblstat(chbufp, &c1_t_mcyc) == ERROR_NO_SYNC)
	{
	/* Invalid interruption!! */
	/* Do not syncronize to system tasks. */
		yns_end_link_acc(chbufp);/* Terminate cyclic communication */
		return ERROR_NO_SYNC;
	}

/* JL-080 status error(ESTS) check */
	if((ests = yns_read_comstat(chbufp)) != 0)
	{
		if(ests & YN_ESTS_SYS_ERR)
		{	/* JL-080 System error!! */
			yns_end_link_acc(chbufp);
			return ERROR_JL080_SYSTEM;	/* Stop cyclic communication */
		}
		else
		{	/* JL-080 Application(user) error!! */
			yns_end_link_acc(chbufp);
			return ERROR_JL080_APPL;	/* Stop cyclic communication */
		}
	}

/* Receive Process [JL-080->buffer] */
	if(yns_read_ldata(chbufp, 1, lnk_rbuff, LINK_SIZE) < 0)
	{
		/* Receive Error!! */
		ret = ERROR_RECV_DATA;
	}

/* Send Process [buffer->JL-080] */
	yns_write_ldata(chbufp, 0, lnk_sbuff, LINK_SIZE);

/* Finish link read/write access */
	yns_end_link_acc(chbufp);

	return ret;
}
